#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-video
# Created by the Natural History Museum in London, UK
import pytest
from ckan.plugins import toolkit
from ckan.tests import factories
from ckan.tests.helpers import call_action

VIDEO_URL = 'https://www.youtube.com/embed/-voP-V7YcqA'
TITLE = 'Test View'
DESCRIPTION = 'A nice test view'
WIDTH = 400
HEIGHT = 400


@pytest.fixture
def rendered_view(app):
    package = factories.Dataset()
    resource = factories.Resource(package_id=package['id'])
    data_dict = {
        'resource_id': resource['id'],
        'view_type': 'video',
        'title': TITLE,
        'description': DESCRIPTION,
        'height': WIDTH,
        'width': HEIGHT,
        'video_url': VIDEO_URL,
    }
    call_action('resource_view_create', **data_dict)
    url = toolkit.url_for(
        'resource.read', id=package['name'], resource_id=resource['id']
    )
    return app.get(url)


@pytest.mark.filterwarnings('ignore::sqlalchemy.exc.SADeprecationWarning')
@pytest.mark.ckan_config('ckan.plugins', 'video')
@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestVideoView(object):
    def test_video_is_shown(self, rendered_view):
        assert VIDEO_URL in rendered_view

    def test_title_description_iframe_shown(self, rendered_view):
        assert TITLE in rendered_view
        assert DESCRIPTION in rendered_view
        assert 'iframe' in rendered_view

    def test_iframe_attributes(self, rendered_view):
        assert f'width="{WIDTH}"' in rendered_view
        assert f'height="{HEIGHT}"' in rendered_view



# !/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-video
# Created by the Natural History Museum in London, UK

video_provider_patterns = {
    'vimeo': '//player.vimeo.com/video/[0-9]+',  # Embedded vimeo link
    'youtube_link': 'http[s]?://www.youtube.com/watch\?v=([0-9a-z/-]+)',
    # Link to you tube video page - () group to de-mark video ID
    'youtube_embedded': 'http[s]?://www.youtube.com/embed/([0-9a-z/-]+)',
    # Embedded youtube video - () group to de-mark video ID
}



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-video
# Created by the Natural History Museum in London, UK

import logging
import re

from ckan.plugins import SingletonPlugin, implements, interfaces, toolkit

from ckanext.video.logic.validators import is_valid_video_url
from ckanext.video.providers import video_provider_patterns

log = logging.getLogger(__name__)
ignore_empty = toolkit.get_validator('ignore_empty')
not_empty = toolkit.get_validator('not_empty')
is_positive_integer = toolkit.get_validator('is_positive_integer')


class VideoPlugin(SingletonPlugin):
    """
    Resource view for embedding videos (youtube/vimeo)
    """

    implements(interfaces.IConfigurer, inherit=True)
    implements(interfaces.IResourceView, inherit=True)
    implements(interfaces.IPackageController, inherit=True)

    def update_config(self, config):
        toolkit.add_template_directory(config, 'theme/templates')

    def info(self):
        return {
            'name': 'video',
            'title': 'Embedded video',
            'schema': {
                'video_url': [ignore_empty, str, is_valid_video_url],
                'width': [not_empty, is_positive_integer],
                'height': [not_empty, is_positive_integer],
            },
            'iframed': False,
            'icon': 'film',
        }

    def can_view(self, data_dict):
        return True

    def view_template(self, context, data_dict):
        return 'video_view.html'

    def form_template(self, context, data_dict):
        return 'video_form.html'

    def setup_template_variables(self, context, data_dict):
        """
        Setup variables available to templates.

        :param context:
        :param data_dict:
        """
        video_url = data_dict['resource_view'].get('video_url') or data_dict[
            'resource'
        ].get('url')

        # Is this a youtube video?
        if 'youtube.com' in video_url:
            # If this is a youtube link, replace with a URL for embeddable video
            match = re.search(
                video_provider_patterns['youtube_link'], video_url, re.IGNORECASE
            )
            if match:
                video_url = f'https://www.youtube.com/embed/{match.group(1)}'

        # TODO - More video provider types

        return {'defaults': {'width': 480, 'height': 390}, 'video_url': video_url}



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-video
# Created by the Natural History Museum in London, UK

import re

from ckan.plugins import toolkit

from ckanext.video.providers import video_provider_patterns


def is_valid_video_url(value, context):
    """
    Validate a URL is a valid video URL.

    :param value:
    :param context:
    """
    for pattern in video_provider_patterns.values():
        if re.search(pattern, value, re.IGNORECASE):
            return value

    raise toolkit.Invalid(toolkit._('URL is not a valid video provider'))



import pytest
from ckan.plugins import toolkit
from mock import MagicMock

from ckanext.video.logic.validators import is_valid_video_url


class TestValidVideoURL(object):
    def test_vimeo_valid(self):
        assert is_valid_video_url(
            'https://player.vimeo.com/video/459651092', MagicMock()
        )
        assert is_valid_video_url('//player.vimeo.com/video/459651092', MagicMock())

    def test_vimeo_invalid(self):
        with pytest.raises(toolkit.Invalid):
            is_valid_video_url('https://player.vimeo.com/video/', MagicMock())
        with pytest.raises(toolkit.Invalid):
            is_valid_video_url('https://vimeo.com/459651092', MagicMock())

    def test_youtube_valid(self):
        assert is_valid_video_url(
            'https://www.youtube.com/watch?v=-voP-V7YcqA', MagicMock()
        )

    def test_youtube_invalid(self):
        with pytest.raises(toolkit.Invalid):
            is_valid_video_url('https://www.youtube.com/watch?x=beans', MagicMock())
        with pytest.raises(toolkit.Invalid):
            is_valid_video_url('https://www.youtube.com/watch?v=', MagicMock())

    def test_youtube_embed_valid(self):
        assert is_valid_video_url(
            'https://www.youtube.com/embed/-voP-V7YcqA', MagicMock()
        )

    def test_youtube_embed_invalid(self):
        with pytest.raises(toolkit.Invalid):
            is_valid_video_url('https://www.youtube.com/embed/', MagicMock())



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-video
# Created by the Natural History Museum in London, UK

import setuptools

if __name__ == '__main__':
    setuptools.setup()
